/*
    libodsstream is a library to read and write ODS documents as streams
    Copyright (C) 2013  Olivier Langella <Olivier.Langella@moulon.inra.fr>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef MANIFESTXML_H
#define MANIFESTXML_H

#include <quazip/quazip.h>
#include <QXmlStreamWriter>

class ManifestXml
{

public:
    ManifestXml(QuaZip * p_quaZip);
    virtual ~ManifestXml();
private:
    static QString _namespaceURI;
    void WriteHeader();

    QuaZip * _p_quaZip;
    QXmlStreamWriter * _p_writer;
};

#endif // MANIFESTXML_H
